library easy_ads;

import 'package:firebase_admob/firebase_admob.dart';
import 'package:static_sharedpreferences/static_sharedpreferences.dart';

import 'banner_ad/easy_banner_ad.dart';
import 'interstitial_ad/easy_interstital_ad.dart';
import 'rewarded_video_ad/easy_rewarded_video_ad.dart';

/// ## Description
///
/// * Support for Google AdMob mobile ads.
///
///
/// * Before initializing, loading or showing an ad the plugin `must` be initialized
///   {@tool sample}
///   ```
///   void main(){
///     WidgetsFlutterBinding.ensureInitialized();
///
///     EasyAds.initialize();
///
///     EasyAds.initEasyBannerAd(myBannerAd);
///
///     runApp(MyApp);
///   }
///   ```
///  {@end-tool}
///
/// ## See also:
///
///  * [EasyBannerAd], a small rectangular ad displayed at the bottom of the screen.
///  * [EasyInterstitialAd], a full screen ad that must be dismissed by the user.
///  * [EasyRewardedVideoAd], a full screen video ad that provides in-app user
///    rewards.
abstract class EasyAds {
  /// Part of the key, which will be used for
  /// all the SharedPreferences variables in
  /// the EasyAds package
  static const String SP_KEY = "EasyAds";

  static bool _isInitializedFirebaseAdMob = false;
  static bool get isInitialized => _isInitializedFirebaseAdMob;

  /// In case you do not give your Admob ID
  /// as a parameter, then the platform specific
  /// testAppId will be used:
  ///
  /// ```
  /// Android: 'ca-app-pub-3940256099942544~3347511713'
  ///
  /// IOS: 'ca-app-pub-3940256099942544~1458002511'
  /// ```
  static void initialize({
    String admobId,
    String trackingId,
    bool analyticsEnabled = false,
  }) {
    SP.initialize();

    admobId = admobId ?? FirebaseAdMob.testAppId;
    FirebaseAdMob.instance
        .initialize(
      appId: admobId,
      trackingId: trackingId,
      analyticsEnabled: analyticsEnabled,
    )
        .then((onValue) {
      _isInitializedFirebaseAdMob = true;
    });
  }

  static const MobileAdTargetingInfo defaultTargetingInfo =
      MobileAdTargetingInfo(childDirected: false);

  static void setOnSharedPreferencesIsNull(
          Function onSharedPreferencesIsNull) =>
      SP.onSharedPreferencesIsNull = onSharedPreferencesIsNull;

  ///EasyBannerAd ----------------------------------------------------------

  static EasyBannerAd _bannerAd;

  static EasyBannerAd get easyBannerAd => _bannerAd;

  static void initEasyBannerAd(EasyBannerAd bannerAd) {
    _bannerAd = bannerAd;
  }

  static Future<bool> loadEasyBannerAd() => _bannerAd.load();

  static Future<bool> showEasyBannerAd({
    double anchorOffset = 0.0,
    double horizontalCenterOffset = 0.0,
    AnchorType anchorType = AnchorType.bottom,
  }) =>
      _bannerAd.show(
        anchorOffset: anchorOffset,
        horizontalCenterOffset: horizontalCenterOffset,
        anchorType: anchorType,
      );

  static Future<bool> disposeEasyBannerAd() => _bannerAd.dispose();

  ///EasyInterstitialAd ----------------------------------------------------

  static EasyInterstitialAd _interstitialAd;

  static EasyInterstitialAd get easyInterstitialAd => _interstitialAd;

  static void initEasyInterstitialAd(EasyInterstitialAd interstitialAd) {
    _interstitialAd = interstitialAd;
  }

  static Future<bool> loadEasyInterstitialAd() => _interstitialAd.load();

  static Future<bool> showEasyInterstitialAd({
    double anchorOffset = 0.0,
    double horizontalCenterOffset = 0.0,
    AnchorType anchorType = AnchorType.bottom,
  }) =>
      _interstitialAd.show(
        anchorOffset: anchorOffset,
        horizontalCenterOffset: horizontalCenterOffset,
        anchorType: anchorType,
      );

  static Future<bool> disposeEasyInterstitialAd() => _interstitialAd.dispose();

  ///EasyRewardedVideoAd ---------------------------------------------------

  static EasyRewardedVideoAd _videoAd;

  static EasyRewardedVideoAd get easyRewardedVideoAd => _videoAd;

  static void initEasyRewardedVideoAd(EasyRewardedVideoAd videoAd) {
    _videoAd = videoAd;
  }

  static Future<bool> loadEasyRewardedVideoAd(
          {MobileAdTargetingInfo targetingInfo}) =>
      (targetingInfo != null)
          ? _videoAd.load(mobileAdtargetingInfo: targetingInfo)
          : _videoAd.load();

  static Future<bool> showEasyRewardedVideoAd() => _videoAd.show();
}

enum Ad_Type {
  VIDEO,
  BANNER,
  INTERSTITIAL,
}

typedef AdLoadedFunction = void Function();
typedef AdClickedFunction = void Function();
typedef AdImpressionFunction = void Function();
typedef AdOpenedFunction = void Function();
typedef AdStartedFunction = void Function();
typedef AdRewardedFunction = void Function(
    int rewardedAmount, String rewardedType);
typedef AdClosedFunction = void Function();
typedef AdLeftApplicationFunction = void Function();
typedef AdFailedToLoadFunction = void Function();
typedef AdCompletedFunction = void Function();
typedef VideoAdEventTriggeredFunction = void Function(
    RewardedVideoAdEvent event);
typedef MobileAdEventTriggeredFunction = void Function(MobileAdEvent event);
